@@ -336,9 +336,23 @@ class dyagram:
             # if not CONN_SET:
             #     dev = ConnectHandler(**netmiko_args)
             self.log.info(f"DEVICE: {device} - SSH : CREATING NETMIKO OBJ : discover_routes_ssh")
-            dev = ConnectHandler(**netmiko_args)
+            tries = 0
+            while tries < 10: #FIXME: THIS SHOULD TRY 10 TIMES THEN RETURN UNABLE TO RUN DYAGRAM
+                try:
+                    dev = ConnectHandler(**netmiko_args)
+                    dev.enable()
+                    tries = 10  # break
+                except:
+                    self.log.info(
+                        f"DEVICE: {device} - SSH : FAILED TO CREATE NETMIKO OBJ, TRY AGAIN: discover_routes_ssh")
+                    time.sleep(1)
+                    tries += 1
+                    if tries == 10:
+                        self.log.info(f"DEVICE: {device} Unable to connect. Please, resolve and re-run Dyagram Discover.") #  CREATE FUNCTION THAT STOPS PROGRAM AND PRINTS SCREEN
+
+
             self.log.info(f"DEVICE: {device} - SSH : CREATED NETMIKO OBJ : discover_routes_ssh")
-            dev.enable()
+
 
             routes_w_vrf = dev.send_command("show ip route vrf all", use_textfsm=True)
             routes_wo_vrf = dev.send_command("show ip route", use_textfsm=True)
@@ -370,7 +384,7 @@ class dyagram:
                            # this is due to deeper dictionary items
 
         diff_resp = DeepDiff(previous_state, current_state, ignore_order=True)
-        print(f"DIFF RESPONSE: {diff_resp}")
+        print(diff_resp)
         if not diff_resp:
             return None
 
@@ -512,6 +526,9 @@ class dyagram:
                         self.log.info(f"DEVICE: {device} EXCEPTION THROWN IN GET_DEVICE_TYPE: {tb}")
                         raise Exception("Unable to access device")
                     autodetect_tries += 1
+                    if autodetect_tries == 10:
+                        self.log.info(f"DEVICE: {device} Unable to connect. Please, resolve and re-run Dyagram Discover.") #  CREATE FUNCTION THAT STOPS PROGRAM AND PRINTS SCREEN
+
                     time.sleep(5)
         except:
             tb = self.get_traceback()
@@ -595,10 +612,23 @@ class dyagram:
                 self.log.info("GOT DEVICE TYPE IN EIGRP NEIGHBORS SSH")
 
             self.log.info("CREATING NETMIKO OBJ : discover_eigrp_neighbors_ssh")
-            dev = ConnectHandler(**nm_args)
-            self.log.info("CREATED NETMIKO OBJ : discover_eigrp_neighbors_ssh")
 
-            dev.enable()
+
+            tries = 0
+            while tries < 10:  # FIXME: THIS SHOULD TRY 10 TIMES THEN RETURN UNABLE TO RUN DYAGRAM
+                try:
+                    dev = ConnectHandler(**nm_args)
+                    dev.enable()
+                    tries = 10  # break
+                except:
+                    self.log.info(
+                        f"DEVICE: {device} - SSH : FAILED TO CREATE NETMIKO OBJ, TRY AGAIN: discover_routes_ssh")
+                    time.sleep(1)
+                    tries += 1
+                    if tries == 10:
+                        self.log.info(f"DEVICE: {device} Unable to connect. Please, resolve and re-run Dyagram Discover.") #  CREATE FUNCTION THAT STOPS PROGRAM AND PRINTS SCREEN
+
+            self.log.info(f"DEVICE: {device} CREATED NETMIKO OBJ : discover_eigrp_neighbors_ssh")
 
 
             if os in ['cisco_ios', 'cisco_nxos']:
@@ -708,7 +738,7 @@ class dyagram:
             self.log.info(f"DEVICE: {device} - SSH : CREATING NETMIKO OBJ : _discover_lldp_neighbors_by_ssh")
 
             tries = 0
-            while tries < 10:
+            while tries < 10: # FIXME: THIS SHOULD TRY 10 TIMES THEN RETURN UNABLE TO RUN DYAGRAM
                 try:
                     dev = ConnectHandler(**netmiko_args)
                     tries = 10 # break
@@ -716,6 +746,9 @@ class dyagram:
                     self.log.info(f"DEVICE: {device} - SSH : FAILED TO CREATE NETMIKO OBJ, TRY AGAIN: _discover_lldp_neighbors_by_ssh")
                     time.sleep(1)
                     tries += 1
+                    if tries == 10:
+                        self.log.info(f"DEVICE: {device} Unable to connect. Please, resolve and re-run Dyagram Discover.") #  CREATE FUNCTION THAT STOPS PROGRAM AND PRINTS SCREEN
+
 
 
             self.log.info(f"DEVICE: {device} - SSH : CREATED NETMIKO OBJ : _discover_lldp_neighbors_by_ssh")
@@ -955,52 +988,60 @@ class dyagram:
         return cdp_info_json
 
     def _get_hostname(self, netmiko_session=None, restconf_session=None):
+        got_hostname = False
+        tries = 0
+        while not got_hostname and tries < 10:
+            try:
+                if not netmiko_session and restconf_session:
+                    try:
+                        device = re.search('\d+\.\d+\.\d+\.\d+', restconf_session.base_url).group(1)
+                    except:
+                        device = restconf_session.base_url
 
-        try:
-            if not netmiko_session and restconf_session:
-                try:
-                    device = re.search('\d+\.\d+\.\d+\.\d+', restconf_session.base_url).group(1)
-                except:
-                    device = restconf_session.base_url
-
-                try:
-                    self.log.info(f"DEVICE: {device} - RESTCONF_OPENCONFIG : Querying for hostname - START")
-                    resp = restconf_session.get(f"{device}/openconfig-system:system/config/name")
-                    if resp.status_code == 200:
-                        self.log.info(f"DEVICE: {device} - RESTCONF_OPENCONFIG : Querying hostname - SUCCESSFUL")
-                        return resp.json()['hostname']
-                    if resp.status_code != 200:
-                        self.log.info(
-                            f"DEVICE: {device} - RESTCONF_OPENCONFIG : Querying hostname - FAILURE")
-                        # try nexus TEMPORARY
-                        self.log.info(
-                            f"DEVICE: {device} - RESTCONF_NXOS_OS_DEVICE_YANG : Querying hostname - START")
-                        resp = restconf_session.get(f"{device}/Cisco-NX-OS-device:System/name")
-
+                    try:
+                        self.log.info(f"DEVICE: {device} - RESTCONF_OPENCONFIG : Querying for hostname - START")
+                        resp = restconf_session.get(f"{device}/openconfig-system:system/config/name")
                         if resp.status_code == 200:
+                            self.log.info(f"DEVICE: {device} - RESTCONF_OPENCONFIG : Querying hostname - SUCCESSFUL")
+                            return resp.json()['hostname']
+                        if resp.status_code != 200:
                             self.log.info(
-                                f"DEVICE: {device} - RESTCONF_NXOS_OS_DEVICE_YANG : Querying hostname - SUCCESSFUL")
-                            return resp.json()['name']
+                                f"DEVICE: {device} - RESTCONF_OPENCONFIG : Querying hostname - FAILURE")
+                            # try nexus TEMPORARY
+                            self.log.info(
+                                f"DEVICE: {device} - RESTCONF_NXOS_OS_DEVICE_YANG : Querying hostname - START")
+                            resp = restconf_session.get(f"{device}/Cisco-NX-OS-device:System/name")
+
+                            if resp.status_code == 200:
+                                self.log.info(
+                                    f"DEVICE: {device} - RESTCONF_NXOS_OS_DEVICE_YANG : Querying hostname - SUCCESSFUL")
+                                return resp.json()['name']
+                            self.log.info(
+                                f"DEVICE: {device} - RESTCONF_NXOS_OS_DEVICE_YANG : Querying hostname - FAILURE")
+
+                    except:
+
+                        tb = self.get_traceback()
                         self.log.info(
-                            f"DEVICE: {device} - RESTCONF_NXOS_OS_DEVICE_YANG : Querying hostname - FAILURE")
+                            f"DEVICE: {device} - RESTCONF_OPENCONFIG_AND_NXOS_OS_DEVICE_YANG_CATCHALL : Querying hostname - FAILURE - TB: {tb}")
 
-                except:
+                else:
+                    self.log.info(f"DEVICE: {netmiko_session.host} - SSH : Querying for hostname - START")
+                    sh_run_output = netmiko_session.send_command("sh run | inc hostname")
+                    hostname = re.search("hostname\s+(.*)", sh_run_output).group(1)
+                    self.log.info(f"DEVICE: {netmiko_session.host} - SSH : Querying for hostname - SUCCESSFUL")
+                    return hostname
+                return None
 
-                    tb = self.get_traceback()
+            except:
+                tries += 1
+                tb = self.get_traceback()
+                self.log.info(f"DEVICE: {device} EXCEPTION THROWN IN _get_hostname, TRY AGAIN : {tb}")
+                if tries == 10:
                     self.log.info(
-                        f"DEVICE: {device} - RESTCONF_OPENCONFIG_AND_NXOS_OS_DEVICE_YANG_CATCHALL : Querying hostname - FAILURE - TB: {tb}")
+                        f"DEVICE: {device} Unable to connect. Please, resolve and re-run Dyagram Discover.")  # CREATE FUNCTION THAT STOPS PROGRAM AND PRINTS SCREEN
 
-            else:
-                self.log.info(f"DEVICE: {netmiko_session.host} - SSH : Querying for hostname - START")
-                sh_run_output = netmiko_session.send_command("sh run | inc hostname")
-                hostname = re.search("hostname\s+(.*)", sh_run_output).group(1)
-                self.log.info(f"DEVICE: {netmiko_session.host} - SSH : Querying for hostname - SUCCESSFUL")
-
-                return hostname
-            return None
-        except:
-            tb = self.get_traceback()
-            self.log.info(f"DEVICE: {device} EXCEPTION THROWN IN _get_hostname : {tb}")
+                time.sleep(1)
 
 
     def _get_os_version(self, netmiko_session):
